// (C) Marc Prager, 2010
#ifndef __string_h
#define __string_h

#include <stddef.h>

/** Copy non-overlapping memory regions.
 * Implementation detail1: uses stack.
 * Implementation detail2: src and dst may overlap, if src<dst (non-portable!).
 * @param dst destination address.
 * @param src source address.
 * @param n number of bytes to copy.
 * @return the original value of dst
 */
void* memcpy(void * restrict dst, const void * restrict src, size_t n);

/** Copy possibly overlapping memory regions.
 * Implementation detail1: uses stack.
 * @param dst destination address.
 * @param src source address.
 * @param n number of bytes to copy.
 * @return the original value of dst
 */
void* memmove(void *dst, const void *src, size_t n);

/** Fill memory area with a specified value.
 * Implementation detail: does NOT use stack.
 * @param dst destination address
 * @param value fill value, only the lower 8 bits are used.
 * @param n number of bytes to set.
 * @return the original value of dst
 */
void* memset(void *dst, int value, size_t n);


/** Compare possibly overlapping memory regions.
 * Implementation detail1: uses stack.
 * @param s1 source address 1
 * @param s2 source address 2
 * @param n number of bytes to compare at most.
 * @return 's1 - s2', i.e. 0 if equal, <0 if s1's values are less than s2's, >0 otherwise
 */
int memcmp(const void *s1, const void *s2, size_t n);

/** Calculate string length.
 * @param s a 0-terminated string
 * @return length in characters
 */
size_t strlen(const char *s);

/** Search for first occurrence of a char in a string. NOTE: this prototype differs from the standard type.
 * @param str the string to be searched.
 * @param c the character to search for.
 * @return a pointer to the position of c inside str or 0 if c cannot be found.
 */
const char* strchr(const char *str, char c);

#endif
