/*
  armv6-m/tl.h 
  Copyright 2014 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

.arch armv6-m
.syntax unified
.equ	_stackThread,	0
.equ	_stackCaller,	4

.global tlCreate, tlYield, tlCall

.text
.thumb

/* ARM AAPCS, rev 2.09, 2012:
 * r0..r3 : parameter passing
 * r0,r1  : return value passing
 * r0..r3 : local variable - between calls (no need to save)
 * r4..r11 : callee preserved
 * r12=ip : intra procedure scratch register - between calls (no need to save)
 */

.func
/** void tlCreate(TlContext tlc, void (*loop)(TlContext tlc));
 * Initializes the caller stack, then starts the loop which then yields back to this function.
 */
tlCreate:
@ param r0: context structure
@ param r1: loop function
	@stmfd sp!,{r4-r11,lr}		@ lr is the caller's return address - we will return there
	push {r4-r7,lr}
		mov r4,r8
		mov r5,r9
		mov r6,r10
		mov r7,r11
	push {r4-r7}

	mov r2,sp
	str r2,[r0,#_stackCaller]	@ save caller's SP

	ldr r2,[r0,#_stackThread]	@ load the threads SP - empty stack
	mov sp,r2

	bx r1				@ jump to thread function; that one MUST NOT return!
.endfunc

.func
/* The thread called yield 
 * R0 = context structure
 * LR = return point to the thread
 * SP = current thread's stack
 */
tlYield:
					@ lr = threads continuation
	@stmfd sp!,{r4-r11,lr}
	push {r4-r7,lr}
		mov r4,r8
		mov r5,r9
		mov r6,r10
		mov r7,r11
	push {r4-r7}

	mov r2,sp
	str r2,[r0,#_stackThread]	@ save thread's SP and registers

	ldr r2,[r0,#_stackCaller]	@ restore callers SP
	mov sp,r2

	@ldmfd sp!,{r4-r11,lr}		@ restore callers' registers for return of tlCall()
	pop {r4-r7}
		mov r11,r7
		mov r10,r6
		mov r9,r5
		mov r8,r4
	pop {r4-r7,pc}			@ implicit return
.endfunc

.func
/** The caller's invokation of the thread.
 */
tlCall:
	@ stmfd sp!,{r4-r11,lr}		@ save own registers (caller)
	push {r4-r7,lr}
		mov r4,r8
		mov r5,r9
		mov r6,r10
		mov r7,r11
	push {r4-r7}

	mov r2,sp
	str r2,[r0,#_stackCaller]	@ save own SP

	ldr r2,[r0,#_stackThread]	@ load thread SP
	mov sp,r2

	@ ldmfd sp!,{r4-r11,lr}		@ restore threads' registers for return of tlYield()
	pop {r4-r7}
		mov r11,r7
		mov r10,r6
		mov r9,r5
		mov r8,r4
	pop {r4-r7,pc}
.endfunc

.end
