/*
  lpc8/hrTimer.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__hrTimer_h
#define lpc8__hrTimer_h

/** @file
 * @brief High resolution timer based on MRT channel 3 (replacement for usTimer).
 */
#include <lpc8/mrt.h>


inline static void hrTimerInit(void) {
	MRT.ctrl3	= 0<<_MRT_CTRL_INTEN | 0<<_MRT_CTRL_MODE;	// free running, no int, repetitive mode
	MRT.intval3	= 1<<30 | 1<<_MRT_INTVAL_LOAD;			// start NOW, cycle = 2^30
}

/** timer using increments of PCLK*4. That makes a wraparound at every 89s @ 12MHz or 35s @ 30MHz.
 */
inline static int hrTimer(void) {
	return (-MRT.timer3)<<2;
}

inline static void hrTimerDelay(int ticks) {
	const int t1 = hrTimer() + ticks;
	while (hrTimer()-t1<0) ;	 // this is NOT the same (WRONG!): while (hrTimer()<t1) ;
}

#endif

