/*
  lpc8/iocon.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */
#ifndef lpc8__iocon_h
#define lpc8__iocon_h

#include <integers.h>
#include <unionized.h>


/** Base address 0x40044000
 */
union PACKED4 Iocon {
	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint32		pLinear[17];	// linear array
	unionized };

	//.0x00
	unionized struct PACKED { char _1[0x00]; // padding
	volatile Uint16		p0_17;
	unionized };
	//.0x04
	unionized struct PACKED { char _2[0x04]; // padding
	volatile Uint16		p0_13;
	unionized };
	//.0x08
	unionized struct PACKED { char _3[0x08]; // padding
	volatile Uint16		p0_12;
	unionized };
	//.0x0C
	unionized struct PACKED { char _4[0x0C]; // padding
	volatile Uint16		p0_5;
	unionized };
	//.0x10
	unionized struct PACKED { char _5[0x10]; // padding
	volatile Uint16		p0_4;
	unionized };
	//.0x14
	unionized struct PACKED { char _6[0x14]; // padding
	volatile Uint16		p0_3;
	unionized };
	//.0x18
	unionized struct PACKED { char _7[0x18]; // padding
	volatile Uint16		p0_2;
	unionized };
	//.0x1C
	unionized struct PACKED { char _8[0x1C]; // padding
	volatile Uint16		p0_11;
	unionized };
	//.0x20
	unionized struct PACKED { char _9[0x20]; // padding
	volatile Uint16		p0_10;
	unionized };
	//.0x24
	unionized struct PACKED { char _10[0x24]; // padding
	volatile Uint16		p0_16;
	unionized };
	//.0x28
	unionized struct PACKED { char _11[0x28]; // padding
	volatile Uint16		p0_15;
	unionized };
	//.0x2C
	unionized struct PACKED { char _12[0x2C]; // padding
	volatile Uint16		p0_1;
	unionized };
	//.0x34
	unionized struct PACKED { char _13[0x34]; // padding
	volatile Uint16		p0_9;
	unionized };
	//.0x38
	unionized struct PACKED { char _14[0x38]; // padding
	volatile Uint16		p0_8;
	unionized };
	//.0x3C
	unionized struct PACKED { char _15[0x3C]; // padding
	volatile Uint16		p0_7;
	unionized };
	//.0x40
	unionized struct PACKED { char _16[0x40]; // padding
	volatile Uint16		p0_6;
	unionized };
	//.0x44
	unionized struct PACKED { char _17[0x44]; // padding
	volatile Uint16		p0_0;
	unionized };
	//.0x48
	unionized struct PACKED { char _18[0x48]; // padding
	volatile Uint16		p0_14;
	unionized };
};

extern union Iocon IOCON;

enum {
	_IOCON_MODE	=3,		// 2 bits
		IOCON_MODE_Z_		=0,
		IOCON_MODE_PULLDOWN_	=1,
		IOCON_MODE_PULLUP_	=2,
		IOCON_MODE_REPEATER_	=3,
	_IOCON_HYS	=5,		// 1 bit, 1=hysteresis enable
	_IOCON_INV	=6,		// 1 bit, 1=invert input
	_IOCON_OD	=10,		// 1 bit, 1=open drain enable
	_IOCON_SMODE	=11,		// 2 bits, pulse reject: 0=0 clocks.. 3=3 clocks; don't use for wakeup pins!!
	_IOCON_CLKDIV	=13,		// 3 bits, pulse reject clock: 0=IOCONCLK0DIV..6=IOCONCLK6DIV, 7 reserved.

	IOCON_Z			=0,	// HiZ (concerning pulls), no pull, no hyst, nothing
	IOCON_PULLUP		=IOCON_MODE_PULLUP_<<_IOCON_MODE,
	IOCON_PULLDOWN		=IOCON_MODE_PULLDOWN_<<_IOCON_MODE,
	IOCON_MODE_REPEATER	=IOCON_MODE_REPEATER_<<_IOCON_MODE,
};

/** Subscript constants for IOCON.pLinear .
 */
enum IoconPlinear {
	IOCON_PLINEAR_0		=16,
	IOCON_PLINEAR_1		=11,
	IOCON_PLINEAR_2		=6,
	IOCON_PLINEAR_3		=5,
	IOCON_PLINEAR_4		=4,
	IOCON_PLINEAR_5		=3,
	IOCON_PLINEAR_6		=15,
	IOCON_PLINEAR_7		=14,
	IOCON_PLINEAR_8		=13,
	IOCON_PLINEAR_9		=12,
	IOCON_PLINEAR_10	=8,
	IOCON_PLINEAR_11	=7,
	IOCON_PLINEAR_12	=2,
	IOCON_PLINEAR_13	=1,
	IOCON_PLINEAR_14	=17,
	IOCON_PLINEAR_15	=10,
	IOCON_PLINEAR_16	=9,
	IOCON_PLINEAR_17	=0,
};

/** Maps PIO0 bit to the index of the ports control word pLinear in IOCON.
 */
extern const Uint8 ioconPlinear[18];

/** Sets the IO mode of a specified bit of PIO0.
 * @param p0Bit the bit number
 * @param mode the IO mode
 */
static inline void ioconMode(int p0Bit, int mode) {
	IOCON.pLinear[ ioconPlinear[p0Bit] ] = mode;
}

#endif

