/*
  lpc8/pmu.h 
  Copyright 2014 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__pmu_h
#define lpc8__pmu_h

#include <integers.h>

union PACKED4 Pmu {
	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint32		pcon;
	unionized };
	
	//.0x04
	unionized struct PACKED { char _1[0x04]; // padding
	volatile Uint32		gpReg[4];
	unionized };

	//.0x04
	unionized struct PACKED { char _2[0x04]; // padding
	volatile Uint32		gpReg0;
	unionized };
	//.0x08
	unionized struct PACKED { char _3[0x08]; // padding
	volatile Uint32		gpReg1;
	unionized };
	//.0x0C
	unionized struct PACKED { char _4[0x0C]; // padding
	volatile Uint32		gpReg2;
	unionized };
	//.0x10
	unionized struct PACKED { char _5[0x10]; // padding
	volatile Uint32		gpReg3;
	unionized };

	//.0x14
	unionized struct PACKED { char _6[0x14]; // padding
	volatile Uint32		dpdCtrl;
	unionized };
};

extern union Pmu PMU;	///< defined in crt0


enum PmuPconBits {
	_PMU_PCON_PM		=0,		///< Power mode, 2bits
		PMU_PCON_PM_DEFAULT_		=0,	///< device is active or in sleep mode
		PMU_PCON_PM_DEEPSLEEP_		=1,	///< WFI will enter deep-sleep mode
		PMU_PCON_PM_POWERDOWN_		=2,	///< WFI will enter power-down mode
		PMU_PCON_PM_DEEPPOWERDOWN_	=3,	///< WFI will enter deep power-down mode
	_PMU_PCON_NODPD		=3,	///< sticky 1bit: prevents entry into deep power-down if set.
	_PMU_PCON_SLEEPFLAG	=8,	///< read: 1=sleep mode entered; write 1: clear bit
	_PMU_PCON_DPDFLAG	=11,	///< read: 1=deep power down entered; write 1: clear bit
};

enum PmuDpdCtrlBits {
	_PMU_DPDCTRL_WAKEUPHYS		=0,	///< 1=hysteresis enabled for wakeup pin; 0=disabled
	_PMU_DPDCTRL_WAKEPADDISABLE	=1,	///< 1=disable wakeup functionality of wakeup pin; 0=enabled
	_PMU_DPDCTRL_LPOSCEN		=2,	///< 1=enable low-power oscillator for WKT; 0=disable
	_PMU_DPDCTRL_LPOSCDPDEN		=3,	///< 1=enable LPOSC even in deep power-down
};

#endif
