/*
  lpc8/scb.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__scb_h
#define lpc8__scb_h

/** @file
 * @brief Cortex-M0 system control block
 *
 *
 */

#include <integers.h>
#include <unionized.h>

extern union PACKED4 {
	//.0xD00
	unionized struct PACKED { char _0[0xD00]; // padding
	const volatile Uint32		cpuId;	///< 0x410CC200 for M0 r0p0
	unionized };
	//.0xD04
	unionized struct PACKED { char _1[0xD04]; // padding
	volatile Uint32			nmiPendSet;
	unionized };
	//.0xD0C
	unionized struct PACKED { char _2[0xD0C]; // padding
	volatile Uint32			aircr;	///< application interrupt and control state register
	unionized };
	//.0xD10
	unionized struct PACKED { char _3[0xD10]; // padding
	volatile Uint32			scr;	///< system control register (sleep modes...)
	unionized };
	//.0xD14
	unionized struct PACKED { char _4[0xD14]; // padding
	const volatile Uint32		ccr;
	unionized };

} SCB;

enum ScbScrBits {
	_SCB_SCR_SEV_ON_PEND	=4,	///< 1=generate an event for each new interrupt pending
	_SCB_SCR_SLEEP_DEEP	=2,	///< select deep sleep, when sleep mode is entered
	_SCB_SCR_SLEEP_ON_EXIT	=1,	///< 1=enter sleep mode automatically after leaving execption handler
};

#endif

