/*
  lpc8/swm.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__swm_h
#define lpc8__swm_h

/** @file
 * @brief Switch Matrix registers.
 */
#include <integers.h>
#include <unionized.h>

/** Maps moveable functions to pins and enables/disables fixed pin functions.
 */
union PACKED4 Swm {
	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint32		pinAssign[9];
	unionized };

	//.0x00
	unionized struct PACKED { char _1[0x00]; // padding
	volatile Uint32		pinAssign0;	///< moveable functions U0_TXD,U0_RXD,U0_RTS,U0_CTS
	unionized };
	//.0x04
	unionized struct PACKED { char _2[0x04]; // padding
	volatile Uint32		pinAssign1;	///< moveable functions U0_SCLK,U1_TXD,U1_RXD,U1_RTS
	unionized };
	//.0x08
	unionized struct PACKED { char _3[0x08]; // padding
	volatile Uint32		pinAssign2;	///< moveable functions U1_CTS,U1_SCLK,U2_TXD,U2_RXD
	unionized };
	//.0x0C
	unionized struct PACKED { char _4[0x0C]; // padding
	volatile Uint32		pinAssign3;	///< moveable functions U2_RTS,U2_CTS,U2_SCLK,SPI0_SCK
	unionized };
	//.0x10
	unionized struct PACKED { char _5[0x10]; // padding
	volatile Uint32		pinAssign4;	///< moveable functions SPI0_MOSI,SPI0_MISO,SPI0_SSEL,SPI1_SCK
	unionized };
	//.0x14
	unionized struct PACKED { char _6[0x14]; // padding
	volatile Uint32		pinAssign5;	///< moveable functions SPI1_MOSI,SPI1_MISO,SPI1_SSEL,CTIN_0
	unionized };
	//.0x18
	unionized struct PACKED { char _7[0x18]; // padding
	volatile Uint32		pinAssign6;	///< moveable functions CTIN_1,CTIN_2,CTIN_3,CTOUT_0
	unionized };
	//.0x1C
	unionized struct PACKED { char _8[0x1C]; // padding
	volatile Uint32		pinAssign7;	///< moveable functions CTOUT_1,CTOUT_2,CTOUT_3,I2C_SDA
	unionized };
	//.0x20
	unionized struct PACKED { char _9[0x20]; // padding
	volatile Uint32		pinAssign8;	///< moveable functions I2C_SCL,ACMP_O,CLKOUT,GPIO_INT_BMAT
	unionized };

	//.0x1C0
	unionized struct PACKED { char _10[0x1C0]; // padding
	volatile Uint32		pinDisable;
	unionized };
};

extern union Swm SWM;		///< defined in crt0

enum SwmPinAssign0Bits {
	_SWM_PINASSIGN0_U0_TXD		=0,
	_SWM_PINASSIGN0_U0_RXD		=8,
	_SWM_PINASSIGN0_U0_RTS		=16,
	_SWM_PINASSIGN0_U0_CTS		=24,
};

enum SwmPinAssign1Bits {
	_SWM_PINASSIGN1_U0_SCLK		=0,
	_SWM_PINASSIGN1_U1_TXD		=8,
	_SWM_PINASSIGN1_U1_RXD		=16,
	_SWM_PINASSIGN1_U1_RTS		=24,
};

enum SwmPinAssign2Bits {
	_SMW_PINASSIGN2_U1_CTS		=0,
	_SMW_PINASSIGN2_U1_SCLK		=8,
	_SMW_PINASSIGN2_U2_TXD		=16,
	_SMW_PINASSIGN2_U2_RXD		=24,
};

enum SwmPinAssign3Bits {
	_SWM_PINASSIGN3_U2_RTS		=0,
	_SWM_PINASSIGN3_U2_CTS		=8,
	_SWM_PINASSIGN3_U2_SCLK		=16,
	_SWM_PINASSIGN3_SPI0_SCLK	=24,
};

enum SwmPinAssign4Bits {
	_SWM_PINASSIGN4_SPI0_MOSI	=0,
	_SWM_PINASSIGN4_SPI0_MISO	=8,
	_SWM_PINASSIGN4_SPI0_SSEL	=16,
	_SWM_PINASSIGN4_SPI1_SCK	=24,
};

enum SwmPinAssign5Bits {
	_SWM_PINASSIGN5_SPI1_MOSI	=0,
	_SWM_PINASSIGN5_SPI1_MISO	=8,
	_SWM_PINASSIGN5_SPI1_SSEL	=16,
	_SWM_PINASSIGN5_CTIN0		=24,
};

enum SwmPinAssign6Bits {
	_SWM_PINASSIGN6_CTIN1		=0,
	_SWM_PINASSIGN6_CTIN2		=8,
	_SWM_PINASSIGN6_CTIN3		=16,
	_SWM_PINASSIGN6_CTOUT0		=24,
};

enum SwmPinAssign7Bits {
	_SWM_PINASSIGN7_CTOUT1		=0,
	_SWM_PINASSIGN7_CTOUT2		=8,
	_SWM_PINASSIGN7_CTOUT3		=16,
	_SWM_PINASSIGN7_I2C_SDA		=24,
};

enum SwmPinAssign8Bits {
	_SWM_PINASSIGN8_I2C_SCL		=0,
	_SWM_PINASSIGN8_ACMP_0		=8,
	_SWM_PINASSIGN8_CLKOUT		=16,
	_SWM_PINASSIGN8_INT_BMAT	=24,
};

enum SwmPinDisableBits {
	_SWM_PINDISABLE_ACMP_I1		=0,
	_SWM_PINDISABLE_ACMP_I2		=1,
	_SWM_PINDISABLE_SWCLK		=2,
	_SWM_PINDISABLE_SWDIO		=3,
	_SWM_PINDISABLE_XTALIN		=4,
	_SWM_PINDISABLE_XTALOUT		=5,
	_SWM_PINDISABLE_RESET		=6,
	_SWM_PINDISABLE_CLKIN		=7,
	_SWM_PINDISABLE_VDDCMP		=8,
	SWM_PINDISABLE_ALL_EXCEPT_RESET	=0x1FF ^ 1<<_SWM_PINDISABLE_RESET, 
};

enum SwmFunction {
	SWM_FUNCTION_U0_TXD,
	SWM_FUNCTION_U0_RXD,
	SWM_FUNCTION_U0_RTS,
	SWM_FUNCTION_U0_CTS,
	SWM_FUNCTION_U0_SCLK,
	SWM_FUNCTION_U1_TXD,
	SWM_FUNCTION_U1_RXD,
	SWM_FUNCTION_U1_RTS,
	SMW_FUNCTION_U1_CTS,
	SMW_FUNCTION_U1_SCLK,
	SMW_FUNCTION_U2_TXD,
	SMW_FUNCTION_U2_RXD,
	SWM_FUNCTION_U2_RTS,
	SWM_FUNCTION_U2_CTS,
	SWM_FUNCTION_U2_SCLK,
	SWM_FUNCTION_SPI0_SCLK,
	SWM_FUNCTION_SPI0_MOSI,
	SWM_FUNCTION_SPI0_MISO,
	SWM_FUNCTION_SPI0_SSEL,
	SWM_FUNCTION_SPI1_SCK,
	SWM_FUNCTION_SPI1_MOSI,
	SWM_FUNCTION_SPI1_MISO,
	SWM_FUNCTION_SPI1_SSEL,
	SWM_FUNCTION_CTIN0,
	SWM_FUNCTION_CTIN1,
	SWM_FUNCTION_CTIN2,
	SWM_FUNCTION_CTIN3,
	SWM_FUNCTION_CTOUT0,
	SWM_FUNCTION_CTOUT1,
	SWM_FUNCTION_CTOUT2,
	SWM_FUNCTION_CTOUT3,
	SWM_FUNCTION_I2C_SDA,
	SWM_FUNCTION_I2C_SCL,
	SWM_FUNCTION_ACMP_0,
	SWM_FUNCTION_CLKOUT,
	SWM_FUNCTION_INT_BMAT,
};

/** Connects one moveable function to a port 0 pin.
 * @param function a moveable function, see SwmFunctionBits.
 * @param pin bit number of a PIO0 port pin.
 */
inline static void swmPin(int function, int pin) {
	const int reg = function>>2;
	const int bit = (function&3)<<3;
	SWM.pinAssign[reg] = SWM.pinAssign[reg] & ~(0xFF<<bit) | (pin & 0xFF)<<bit;
}

#endif

