/*
  lpc8/systick.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

/** @file
 * @brief Systick timer
 *
 */
#ifndef lpc8__systick_h
#define lpc8__systick_h

#include <integers.h>
#include <unionized.h>

/** Shares the SCB area.
 */
extern union PACKED4 {
	//.0x010
	unionized struct PACKED { char _0[0x010]; // padding
	volatile Uint32		ctrl;
	unionized };
	//.0x014
	unionized struct PACKED { char _1[0x014]; // padding
	volatile Uint32		reload;		///< 24bit reload value
	unionized };
	//.0x018
	unionized struct PACKED { char _2[0x018]; // padding
	volatile Uint32		current;	///< read: current value; write: reset to 0, clear COUNTFLAG
	unionized };
	//.0x01C
	unionized struct PACKED { char _3[0x01C]; // padding
	volatile Uint32		calib;
	unionized };
} SYSTICK;

enum SystickCtrlBits {
	_SYSTICK_CTRL_ENABLE	=0,	///< 1=timer running
	_SYSTICK_CTRL_TICKINT	=1,	///< 1=Systick interrupt enable
	_SYSTICK_CTRL_CLKSOURCE	=2,	///< 1=core clock, 0=ref clock (design dependend: CCLK/2 on LPC800)
	_SYSTICK_CTRL_COUNTFLAG	=16,	///< set, when counter reaches 0, clear on read.
};

enum SystickCalibBits {
	_SYSTICK_CALIB_NOREF	=31,
	_SYSTICK_CALIB_SKEW	=30,	///< 1 (RO) indicates that TENMS is not accurate (design dependend).
	_SYSTICK_CALIB_TENMS	=0,	///< 24 bits value to get a 10ms rate.
};

#endif
