#ifndef __lpc8_systickTimer_h
#define __lpc8_systickTimer_h
/*
  lpc8/systickTimer.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

/** @file
 * @brief An interrupt-driven typically 10ms resolution system time that stops with the core.
 */

#include <integers.h>

/** This variable is incremented by the systick interrupt.
 */
extern volatile int systickTimerMs;

/** Initialize systick timer.
 * A systick handler is installed to update the sysTickTimerMs variable.
 * @param clks the number of CCLK clocks for one timer cycle (interrupt) - typically a period of 10ms (100Hz).
 * @param ms the number of milliseconds that correspond to the time ISR cycle.
 */
void systickTimerInit(int clks, int ms);

/** Convenience busy wait delay function.
 * @param ms the number of milliseconds to wait.
 */
inline static void systickTimerDelayMs(int ms) {
	int t0 = systickTimerMs;
	while (t0+ms>systickTimerMs) ;
}

#endif
