/*
  lpc8/wkt.h 
  Copyright 2013 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

/** @file
 * @brief WKT - wake-up timer and LPOSC (10kHz low power oscillator).
 *
 * The WKT is clocked by IRC or LPOSC. IRC is powered in active mode, in sleep mode and in deep-sleep mode.
 * LPOSC can be active in all modes, depending on PMU.dpdCtrl (_LPOSCEN, _LPOSCDPEN) only.
 */
#ifndef lpc8__wkt_h
#define lpc8__wkt_h

#include <unionized.h>
#include <integers.h>

union PACKED4 Wkt {
	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint32		ctrl;
	unionized };

	//.0x0C
	unionized struct PACKED { char _1[0x0C]; // padding
	volatile Uint32		count;
	unionized };
};

extern union Wkt WKT;

enum WktCtrlBits {
	_WKT_CTRL_CLKSEL		=0,	///< 0=IRC clock (default); 1=Low power clock
		WKT_CTRL_CLKSEL_IRC_	=0,
		WKT_CTRL_CLKSEL_LPC_	=1,	///< WKT control register, see also PMU.dpdCtrl to power it.
	_WKT_CTRL_ALARM			=1,	///< 0=no time-out; 1=time-out happened. Clear by writing 1.
	_WKT_CTRL_CLR			=2,	///< write 1=clear counter (halt)
};

#endif

