/*
  lpc8/wwdt.h 
  Copyright 2014 Marc Prager
 
  This file is part of the c-lpc8 library.
  c-lpc8 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 
  c-lpc8 is published in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License along with c-lpc8.
  If not see <http://www.gnu.org/licenses/>
 */

#ifndef lpc8__wwdt_h
#define lpc8__wwdt_h

#include <integers.h>

union PACKED4 WWDT {
	//.0x00
	unionized struct PACKED { char _0[0x00]; // padding
	volatile Uint32		mod;	///< mode register
	unionized };
	//.0x04
	unionized struct PACKED { char _1[0x04]; // padding
	volatile Uint32		tc;	///< timer constant
	unionized };
	//.0x08
	unionized struct PACKED { char _2[0x08]; // padding
	volatile Uint32		feed;
	unionized };
	//.0x0C
	unionized struct PACKED { char _3[0x0C]; // padding
	const volatile Uint32	tv;	///< timer value, read-only
	unionized };
	//.0x14
	unionized struct PACKED { char _4[0x14]; // padding
	volatile Uint32		warnInt;	///< warning interrupt compare value
	unionized };
	//.0x18
	unionized struct PACKED { char _5[0x18]; // padding
	volatile Uint32		window;		///< watchdog window compare value
	unionized };
};

extern union Wwdt WWDT;	///< defined in crt0

enum WwdtModBits {
	_WWDT_MOD_WDEN		=0,	///< 0=stopped, 1=running
	_WWDT_MOD_WDRESET	=1,	///< sticky 1: 0=no chip reset; 1=chip reset on timeout
	_WWDT_MOD_WDTOF		=2,	///< indicates timeout; clear by software
	_WWDT_MOD_WDINT		=3,	///< warning interrupt flag; clear by software
	_WWDT_MOD_WDPROTECT	=4,	///< 0=tc can be changed any time; 1=tc can be changed below 'window'
	_WWDT_MOD_LOCK		=5,	///< sticky 1: 1=watchdog disabling or power down prevented.
};

#endif
