/* Linker script for C language projects using LPC8xx with at least 4k RAM.

   (C) Marc Prager, 2013
*/

OUTPUT_FORMAT("elf32-littlearm")
OUTPUT_ARCH(arm)
STARTUP("lpc8-rom.o")

MEMORY {
	flash (rx)	: ORIGIN = 0x00000000, LENGTH = 16K	/* Value of LPC812 only */
	ram (rwx)	: ORIGIN = 0x10000000, LENGTH = 4K
}

SECTIONS
{
	.text		: {
			*(.text.vectorTable)	/* My vector table section, 2012 */
			*(.text.startup)	/* GCC >= 4.3.4 place main into this */
			*(.text*)
			__text_end = . ;
			. = ALIGN(4) ;
	} > flash



	.rodata 	: {
			__rodata_start = . ;
			*(.rodata*)
			__rodata_end = . ;
			. = ALIGN(4) ;
	} > flash



	imageLocation	= ABSOLUTE(.) ;		/* current VMA as absolute address (flash) */
	.data		: AT ( imageLocation ) { 
			__data_bottom = . ;
			*(.data)
			. = ALIGN(4) ;
			__data_top = . ;
	} > ram
	__data_romimage	= LOADADDR(.data) ;


	.bss		: AT (ABSOLUTE(__data_top)) {	/* LD remembers LMA position from .data (flash) */
			__bss_bottom = . ;
			*(.bss) *(COMMON)
			. = ALIGN(4) ;
			__bss_top = . ;
	} > ram

	.stack ALIGN(8) : {
			stackMainBottom = . ;
			. = ORIGIN(ram) + LENGTH(ram) - ABSOLUTE(stackMainBottom) ;
			stackMainTop = . ;
	} > ram
}
