/*
A program for
 - blinking a LED
 - setting up PLL to 24MHz
 - outputting main clock
 - printing some characters
*/

.syntax unified
.cpu cortex-m0plus

.section .text.vectorTable
	.word 0x10000400	@ minimum ram of 1kiB
	.word _start

	.org 0x40		@ skip exception vectors
	.org 0x300		@ skip CRP

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
@ some constants
@
@ !!! TAKE CARE not to fill address 0x2FC (CRP) with a dangerous value !!!

@ SYSCON
.equ	SYSCON_BASE,		0x40048000

@ PLL
.equ	_SYSPLLCTRL,		0x08
	.equ	MSEL,	1
	.equ	PSEL,	2
.equ	_SYSPLLSTAT,		0x0C
.equ	_MAINCLKSEL,		0x70
.equ	_MAINCLKUEN,		0x74



.equ	SYSAHBCLKCTRL,		0x40048080
	.equ	CLK_GPIO,	6		@ bit in SYSAHBCLKCTRL
	.equ	CLK_SWM,	7		@ switch matrix clock enable
	.equ	CLK_IOCON,	18		@ bit in SYSAHBCLKCTRL

.equ	CLKOUT_BASE,		SYSCON_BASE+0x0E0
.equ	_CLKOUTSEL,		0x0
.equ	_CLKOUTUEN,		0x4
.equ	_CLKOUTDIV,		0x8

.equ	PD_BASE,		SYSCON_BASE+0x230
	.equ	_PDSLEEPCFG,	0x0
	.equ	_PDAWAKECFG,	0x4
	.equ	_PDRUNCFG,	0x8
@ SWM
.equ	PINENABLE0,	0x4000C1C0	@ for disabling fixed functions
.equ	PINASSIGN_BASE,	0x4000C000
	.equ	_PINASSIGN0,	0x00
	.equ	_PINASSIGN8,	0x20
@.equ	PINASSIGN8,	0x4000C020	@ for setting CLKOUT (bits 16..23)

@ GPIO
.equ	GPIO,		0xA0002000
.equ	DIRP0,		0xA0002000
.equ	SETP0,		0xA0002200
.equ	CLRP0,		0xA0002280
.equ	NOTP0,		0xA0002300

@ USART0
.equ	USART0_BASE,	0x40064000
	.equ	_USARTCFG,		0x00
	.equ	_USARTCTRL,		0x04
	.equ	_USARTSTAT,		0x08
	.equ	_USARTRXDATA,		0x14
	.equ	_USARTTXDATA,		0x1C
	.equ	_USARTBRG,		0x20

.equ	UARTCLKDIV,	0x40048094	@ SYSCON

.equ	LED,		15	@ LED on P0.15
.equ	CLKOUT,		12	@ CLKOUT on P0.12

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
.text

.global _start
.func
.thumb_func
_start:
	@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	@ enable GPIO and switch matrix
	ldr r1,=SYSAHBCLKCTRL
	movs r2,#1<<CLK_GPIO | 1<<CLK_SWM
	ldr r0,[r1]
	orrs r0,r0,r2
	str r0,[r1]

	@ enable CLKOUT
		@ disable fixed functions first
	ldr r1,=PINENABLE0
	ldr r0,=~(1<<6)		@ keep RESET functionality
	str r0,[r1]

	ldr r1,=PINASSIGN_BASE
	ldr r2,=0xFF0cFFFF
	str r2,[r1,#+_PINASSIGN8]
		@ select source and divider
	ldr r1,=CLKOUT_BASE
	movs r0,#3	@ main clock
	str r0,[r1,#+_CLKOUTSEL]
	movs r0,#1
	movs r2,#0
	str r0,[r1,#+_CLKOUTDIV]
	str r2,[r1,#+_CLKOUTUEN]
	str r0,[r1,#+_CLKOUTUEN]

	@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	@ switch on LED
	ldr r0,=1<<LED
	ldr r1,=DIRP0
	str r0,[r1]
	ldr r1,=SETP0
	str r0,[r1]

	@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	@ configure PLL
		@ configure PLL

	movs r0,# MSEL | PSEL<<5
	ldr r1,=SYSCON_BASE
	str r0,[r1,#+_SYSPLLCTRL]

		@ power on PLL
	ldr r1,=PD_BASE
	ldr r0,[r1,#+_PDRUNCFG]
	movs r2,#1<<7			@ PLL is powered down after reset. Bit is SET!
	bics r0,r0,r2
	str r0,[r1,#+_PDRUNCFG]

		@ check lock
	ldr r1,=SYSCON_BASE
	movs r2,#1
	lock:
		ldr r0,[r1,#+_SYSPLLSTAT]
		tst r0,r2
		beq lock		@ wait for PLL lock

		@ connect PLL
	movs r0,#3		@ 0=IRC(default), 1=PLL in, 2=WDT, 3=PLL out
	str r0,[r1,#+_MAINCLKSEL]
	movs r0,#0
	movs r2,#1
	str r0,[r1,#+_MAINCLKUEN]
	str r2,[r1,#+_MAINCLKUEN]

	@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	@ UART 0, same pins as ISP boot loader

		@ enable clock
		ldr r1,=SYSAHBCLKCTRL
		ldr r0,[r1]
		ldr r2,=1<<14
		orrs r0,r0,r2
		str r0,[r1]

		@ peripheral reset is deasserted (1!) by default

		@ configure switch matrix
		ldr r1,=PINASSIGN_BASE
		ldr r0,=0xFFFF0004	@ RXD->P0_0, TXD0->P0_4
		str r0,[r1,#+_PINASSIGN0]

		@ configure clock prescaler
		movs r0,#1	@ 0 means: disabled, 1=divide by 1, ...
		ldr r1,=UARTCLKDIV
		str r0,[r1]

		@ configure baud rate
		ldr r1,=USART0_BASE
		movs r0,#13-1
		str r0,[r1,#+_USARTBRG]

		@configure frame bits
		movs r0,# 1<<0 | 1<<2 | 0<<4 | 0<<6	@ enable, 8bits, no parity, 1 stop 
		str r0,[r1,#+_USARTCFG]
		movs r0,#0
		str r0,[r1,#+_USARTCTRL]		@ these are the defaults

	@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	@ character burst
	
	ldr r7,=USART0_BASE
	movs r6,#'z'
	movs r5,#'A'
	cburst:
		@ check, if USART is ready
		cwait:
			ldr r0,[r7,#+_USARTSTAT]	@ bit #2 is TXRDY, bit #3 is TXIDLE
			lsrs r0,r0,#2+1
			bcc cwait		@ bit not set => buffer full
		str r5,[r7,#+_USARTTXDATA]
		adds r5,r5,#1
		cmp r5,r6
		bls cburst	@ unsigned lower or same

	@ wait a second for last char
	movs r0,#1
	lsls r0,r0,#22
	1:	subs r0,r0,#1
		bne 1b

	@ blink LED
	ldr r0,=1<<LED
	ldr r1,=NOTP0
	ldr r4,=USART0_BASE
	movs r5,#'A'


	@ blink loop
	blink:
		str r0,[r1]
		str r5,[r4,#+_USARTTXDATA]
		ldr r2,=1048576
		movs r3,#1
		loop:
			subs r2,#1
			bne loop
		b blink
	b .
.endfunc
.pool		@ literate pool here

.end
