/* Exploring LPC800 sleep modes.
 *
 * Distinguish:
 * 1) Power-down: output register retention
 * 2) Deep Power-down: outputs tri-stated
 *
 * ToDo:
 * - leave no floating pins (inputs)
 * - this also applies to RXD after disconnect!
 * - disable SWCLK, SWDIO functionality
 * - pull up /RESET and /WAKEUP externally
 */


#include <lpc8/swm.h>
#include <lpc8/usart.h>
#include <lpc8/pmu.h>
#include <lpc8/scb.h>
#include <lpc8/iocon.h>
#include <lpc8/gpio.h>
#include <lpc8/systickTimer.h>

#include <print.h>
#include <macros.h>

void handlerDefault(void) {
	while(1);
}

/* PINS
 * =====
 *
 * P0.1  /BOOT
 * P0.0  RXD (bootloader)
 * P0.4  TXD (bootloader)
 * P0.5  /RESET
 * P0.10 external pullup 1.5k
 * P0.11 external pullup 1.5k
 * P0.15 CLKOUT
 * P0.16 /LED indicator
*/

enum {
	LED	=16,
};

enum {
	PCLK	=12*1000*1000,
};

void main(void) {
	// do some output first:
	SYSCON.ahbClkCtrl	|= 0
				| 1<<_SYSCON_AHBCLKCTRL_USART0
				| 1<<_SYSCON_AHBCLKCTRL_SWM
				| 1<<_SYSCON_AHBCLKCTRL_IOCON
				| 1<<_SYSCON_AHBCLKCTRL_GPIO
				| 1<<_SYSCON_AHBCLKCTRL_WWDT
				;

	SWM.pinDisable	= SWM_PINDISABLE_ALL_EXCEPT_RESET;

	swmPin(SWM_FUNCTION_U0_RXD,0);
	swmPin(SWM_FUNCTION_U0_TXD,4);

	// unused ports: drive low as GPIO
	static const int unu[] = {
		1,
		2,	// fucking SWDIO
		3,	// fucking SWCLK
		4,6,7,8,9,12,13,14,16,17
	};
	for (int i=0; i<ELEMENTS(unu); i++) {
		const int p = unu[i];
		IOCON.pLinear[ ioconPlinear[p] ] = IOCON_Z;
		GPIO.dir0	|= 1<<p;
		GPIO.set0	= 1<<p;
	}

	IOCON.p0_15		= IOCON_Z;
	GPIO.dir0		|= 1<<LED;
	GPIO.set0		= 1<<LED;
	GPIO.clr0		= 1<<LED;

	swmPin(SWM_FUNCTION_CLKOUT,15);

	SYSCON.clkOutSel	= SYSCON_CLKOUTSEL_MAINCLK;
	SYSCON.clkOutUen	= 0;
	SYSCON.clkOutUen	= 1;
	SYSCON.clkOutDiv	= 1;

	systickTimerInit(PCLK/1000,1);
	usart0InitFrom12Mhz(BAUD_115200_FROM_12MHZ);
	print(usart0BlockingWriteChar);
	printString("Running.\n");

	printString("One second..."); systickTimerDelayMs(1000);
	printString("OK.\n");

	printString("Preparing deep power-down.\n");	// 2mA

	// prepare for disconnect:
	IOCON.p0_0		= IOCON_MODE_PULLUP_<<_IOCON_MODE;	// RXD
	IOCON.p0_1		= IOCON_MODE_PULLUP_<<_IOCON_MODE;	// /BOOT

	SYSCON.clkOutSel	= SYSCON_CLKOUTSEL_MAINCLK;
	SYSCON.mainClkUen	= 0;
	SYSCON.mainClkUen	= 1;

	//SYSCON.usartClkDiv	= 0;
	//GPIO.dir0	|=1<<10 | 1<<11;		// I2C OD-pins
	//GPIO.set0	=1<<10 | 1<<11;

	SCB.scr		= 1<<_SCB_SCR_SLEEP_DEEP
			;
	//PMU.dpdCtrl	= 1<<PMU_DPDCTRL_WAKEPADDISABLE;
	//PMU.pcon	= _PMU_PCON_PM_DEFAULT_ << _PMU_PCON_PM;	// 300uA
	//PMU.pcon	= _PMU_PCON_PM_DEEPSLEEP_ << _PMU_PCON_PM;	// 400uA
	//PMU.pcon	= _PMU_PCON_PM_POWERDOWN_ << _PMU_PCON_PM;	// 2uA (now)
	PMU.pcon	= PMU_PCON_PM_DEEPPOWERDOWN_ << _PMU_PCON_PM;	// < 1uA (now)

	asm volatile("WFI");
	while (1);
}

